<?php
function cbp_whitelist()
{
   $default_ip = arg(5);
   if (!filter_var($default_ip, FILTER_VALIDATE_IP))
   {
      $default_ip = '';
   }
   $rows = array();
   $header = array(t('Whitelisted IP addresses'), t('Operations'));
   $result = db_query('SELECT * FROM {cbp_whitelist}');
   foreach ($result as $ip)
   {
      $rows[] = array
      (
         $ip->ip,
         l(t('delete'), "admin/config/people/ip-whitelist/delete/{$ip->ip}"),
      );
   }

   $build['cbp_whitelist_form'] = drupal_get_form('cbp_whitelist_form', $default_ip);
   $build['cbp_whitelist_table'] = array
   (
      '#theme' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => t('No whitelisted IP addresses available.'),
   );

  return $build;
}

/**
 * Define the form for whitelisting IP addresses.
 *
 * @ingroup forms
 * @see system_ip_whitelist_form_validate()
 * @see system_ip_whitelist_form_submit()
 */
function cbp_whitelist_form($form, $form_state, $default_ip)
{
   $form['ip'] = array
   (
      '#title' => t('IP address'),
      '#type' => 'textfield',
      '#size' => 48,
      '#maxlength' => 40,
      '#default_value' => $default_ip,
      '#description' => t('Enter a valid IP address.'),
  );
  $form['actions'] = array('#type' => 'actions');
  $form['actions']['submit'] = array
  (
    '#type' => 'submit',
    '#value' => t('Add'),
  );
  $form['#submit'][] = 'cbp_whitelist_form_submit';
  $form['#validate'][] = 'cbp_whitelist_form_validate';
  return $form;
}

function cbp_whitelist_form_validate($form, &$form_state)
{
   $ip = trim($form_state['values']['ip']);
   if (db_query("SELECT * FROM {cbp_whitelist} WHERE ip = :ip", array(':ip' => $ip))->fetchField())
   {
      form_set_error('ip', t('This IP address is already whitelisted.'));
   }
   elseif (filter_var($ip, FILTER_VALIDATE_IP, FILTER_FLAG_NO_RES_RANGE) == FALSE)
   {
      form_set_error('ip', t('Enter a valid IP address.'));
   }
}

function cbp_whitelist_form_submit($form, &$form_state)
{
   $ip = trim($form_state['values']['ip']);
   db_insert('cbp_whitelist')->fields(array('ip' => $ip))->execute();
   cbp_purge_bans($ip); //This purges both flood AND blocked_ips tables
   drupal_set_message(t('The IP address %ip has been whitelisted.', array('%ip' => $ip)));
   watchdog('crowd brutefore protection', t('The IP address %ip has been whitelisted.', array('%ip' => $ip)));
   $form_state['redirect'] = 'admin/config/people/ip-whitelist';
  return;
}

/**
 * IP deletion confirm page.
 *
 * @see cbp_whitelist_delete_submit()
 */
function cbp_whitelist_delete($form, &$form_state, $ip)
{
   $form['whitelisted_ip'] = array
   (
      '#type' => 'value',
      '#value' => $ip,
  );
  return confirm_form($form, t('Are you sure you want to delete %ip?', array('%ip' => $ip)), 'admin/config/people/ip-whitelist', t('This action cannot be undone.'), t('Delete'), t('Cancel'));
}

/**
 * Process cbp_whitelist_delete form submissions.
 */
function cbp_whitelist_delete_submit($form, &$form_state)
{
   $whitelisted_ip = $form_state['values']['whitelisted_ip'];
   db_delete('cbp_whitelist')->condition('ip', $whitelisted_ip)->execute();
   watchdog('crowd brutefore protection', 'Deleted Whitelisted IP %ip', array('%ip' => $whitelisted_ip));
   drupal_set_message(t('The IP address %ip was deleted.', array('%ip' => $whitelisted_ip)));
   $form_state['redirect'] = 'admin/config/people/ip-whitelist';
}

